from abc import abstractmethod
from typing import Any, Awaitable, Callable, Dict

from .base import Base
from .request import Request
from .response import Response


class BrokerOptions:
    """Base class for broker options."""
    pass


class Broker(Base):
    """Main class used by the broker subclasses.

    Inherits Base

    """
    @abstractmethod
    async def start(self) -> None:
        """Abstract method to start the backend.

        Must be implemented in the subclass

        """
        pass

    @abstractmethod
    async def process_request(self,
                              request: Request,
                              options: Dict[str, Any],
                              callback: Callable[[Request, Response],
                                                 Awaitable[None]]) -> None:
        """Abstract method to process the request.

        Must be implemented in the subclass

        """
        pass
